@@ -22,10 +22,12 @@ extern "C" {
 #endif
 
 extern struct rtnl_act *rtnl_act_alloc(void);
+extern void		rtnl_act_get(struct rtnl_act *);
 extern void		rtnl_act_put(struct rtnl_act *);
 extern int		rtnl_act_build_add_request(struct rtnl_act *, int,
 						   struct nl_msg **);
 extern int		rtnl_act_add(struct nl_sock *, struct rtnl_act *, int);
+extern int		rtnl_act_change(struct nl_sock *, struct rtnl_act *, int);
 
 extern int		rtnl_act_build_change_request(struct rtnl_act *, int,
 						      struct nl_msg **);
