@@ -2,10 +2,11 @@
  * lib/route/pktloc.c     Packet Location Aliasing
  *
  *	This library is free software; you can redistribute it and/or
- *	modify it under the terms of the GNU General Public License as
- *	published by the Free Software Foundation version 2 of the License.
+ *	modify it under the terms of the GNU Lesser General Public
+ *	License as published by the Free Software Foundation version 2.1
+ *	of the License.
  *
- * Copyright (c) 2008-2010 Thomas Graf <tgraf@suug.ch>
+ * Copyright (c) 2008-2013 Thomas Graf <tgraf@suug.ch>
  */
 
 /**
@@ -20,7 +21,7 @@
  * library and provides a well defined set of definitions for most common
  * protocol fields.
  *
- * @subsection pktloc_examples Examples
+ * @section pktloc_examples Examples
  * @par Example 1.1 Looking up a packet location
  * @code
  * struct rtnl_pktloc *loc;
@@ -30,8 +31,8 @@
  * @{
  */
 
-#include <netlink-local.h>
-#include <netlink-tc.h>
+#include <netlink-private/netlink.h>
+#include <netlink-private/tc.h>
 #include <netlink/netlink.h>
 #include <netlink/utils.h>
 #include <netlink/route/pktloc.h>
@@ -45,7 +46,7 @@
 static struct nl_list_head pktloc_name_ht[PKTLOC_NAME_HT_SIZ];
 
 /* djb2 */
-unsigned int pktloc_hash(const char *str)
+static unsigned int pktloc_hash(const char *str)
 {
 	unsigned long hash = 5381;
 	int c;
@@ -89,18 +90,21 @@ static int read_pktlocs(void)
 	YY_BUFFER_STATE buf = NULL;
 	yyscan_t scanner = NULL;
 	static time_t last_read;
-	struct stat st = {0};
+	struct stat st;
 	char *path;
 	int i, err;
 	FILE *fd;
 
-	asprintf(&path, "%s/pktloc", SYSCONFDIR);
+	if (build_sysconf_path(&path, "pktloc") < 0)
+		return -NLE_NOMEM;
 
 	/* if stat fails, just try to read the file */
 	if (stat(path, &st) == 0) {
 		/* Don't re-read file if file is unchanged */
-		if (last_read == st.st_mtime)
-			return 0;
+		if (last_read == st.st_mtime) {
+			err = 0;
+			goto errout;
+		}
 	}
 
 	NL_DBG(2, "Reading packet location file \"%s\"\n", path);
@@ -136,14 +140,13 @@ static int read_pktlocs(void)
 	last_read = st.st_mtime;
 
 errout_scanner:
-	if (scanner)
-		pktloc_lex_destroy(scanner);
+	pktloc_lex_destroy(scanner);
 errout_close:
 	fclose(fd);
 errout:
 	free(path);
 
-	return 0;
+	return err;
 }
 
 /** @endcond */
@@ -151,6 +154,7 @@ errout:
 /**
  * Lookup packet location alias
  * @arg name		Name of packet location.
+ * @arg result		Result pointer
  *
  * Tries to find a matching packet location alias for the supplied
  * packet location name.
