@@ -138,12 +138,13 @@ static int xfrm_sp_clone(struct nl_object *_dst, struct nl_object *_src)
 	return 0;
 }
 
-static int xfrm_sp_compare(struct nl_object *_a, struct nl_object *_b, uint32_t attrs, int flags)
+static uint64_t xfrm_sp_compare(struct nl_object *_a, struct nl_object *_b,
+				uint64_t attrs, int flags)
 {
 	struct xfrmnl_sp* a  =   (struct xfrmnl_sp *) _a;
 	struct xfrmnl_sp* b  =   (struct xfrmnl_sp *) _b;
 	struct xfrmnl_user_tmpl *tmpl_a, *tmpl_b;
-	int diff = 0;
+	uint64_t diff = 0;
 
 #define XFRM_SP_DIFF(ATTR, EXPR) ATTR_DIFF(attrs, XFRM_SP_ATTR_##ATTR, a, b, EXPR)
 	diff |= XFRM_SP_DIFF(SEL,	xfrmnl_sel_cmp(a->sel, b->sel));
@@ -408,7 +409,7 @@ static void xfrm_sp_dump_line(struct nl_object *a, struct nl_dump_params *p)
 		nl_dump_line(p, "\tUser template: \n");
 
 		nl_list_for_each_entry(utmpl, &sp->usertmpl_list, utmpl_list)
-	        xfrmnl_user_tmpl_dump (utmpl, p);
+			xfrmnl_user_tmpl_dump (utmpl, p);
 	}
 
 	if (sp->ce_mask & XFRM_SP_ATTR_MARK)
@@ -749,8 +750,8 @@ int xfrmnl_sp_get_kernel(struct nl_sock* sock, unsigned int index, unsigned int
 	*result = (struct xfrmnl_sp *) obj;
 
 	/* If an object has been returned, we also need to wait for the ACK */
-	 if (err == 0 && obj)
-		 nl_wait_for_ack(sock);
+	if (err == 0 && obj)
+		nl_wait_for_ack(sock);
 
 	return 0;
 }
