@@ -14,7 +14,7 @@
 #include <netlink/cli/link.h>
 
 static int quiet = 0, default_yes = 0, deleted = 0, interactive = 0;
-struct nl_sock *sock;
+static struct nl_sock *sock;
 
 static void print_usage(void)
 {
@@ -70,13 +70,13 @@ int main(int argc, char *argv[])
 	struct rtnl_class *class;
 	struct rtnl_tc *tc;
 	struct nl_cache *link_cache, *class_cache;
- 
+
 	sock = nl_cli_alloc_socket();
 	nl_cli_connect(sock, NETLINK_ROUTE);
 	link_cache = nl_cli_link_alloc_cache(sock);
- 	class = nl_cli_class_alloc();
+	class = nl_cli_class_alloc();
 	tc = (struct rtnl_tc *) class;
- 
+
 	for (;;) {
 		int c, optidx = 0;
 		enum {
@@ -95,7 +95,7 @@ int main(int argc, char *argv[])
 			{ "kind", 1, 0, 'k' },
 			{ 0, 0, 0, 0 }
 		};
-	
+
 		c = getopt_long(argc, argv, "qhvd:p:i:k:", long_opts, &optidx);
 		if (c == -1)
 			break;
@@ -112,7 +112,7 @@ int main(int argc, char *argv[])
 		case 'i': nl_cli_tc_parse_handle(tc, optarg, 0); break;
 		case 'k': nl_cli_tc_parse_kind(tc, optarg); break;
 		}
- 	}
+	}
 
 	if (!rtnl_tc_get_ifindex(tc))
 		nl_cli_fatal(EINVAL, "You must specify a network device (--dev=XXX)");
